@@ -67,11 +67,16 @@ module Hedgehog.Gen (
   -- ** Conditional
   , discard
   , filter
+  , filterT
   , mapMaybe
+  , mapMaybeT
   , just
+  , justT
 
   -- ** Collections
   , maybe
+  , either
+  , either_
   , list
   , seq
   , nonEmpty
@@ -106,4 +111,4 @@ module Hedgehog.Gen (
 import           Hedgehog.Internal.Gen
 import           Hedgehog.Internal.State (sequential, parallel)
 
-import           Prelude hiding (filter, print, maybe, map, seq)
+import           Prelude hiding (either, filter, print, maybe, map, seq)
