@@ -12,6 +12,7 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
@@ -20,6 +21,7 @@
 {-# LANGUAGE TupleSections #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE UndecidableInstances #-} -- Distributive
+
 module Hedgehog.Internal.Property (
   -- * Property
     Property(..)
@@ -71,6 +73,7 @@ module Hedgehog.Internal.Property (
   , (/==)
 
   , eval
+  , evalNF
   , evalM
   , evalIO
   , evalEither
@@ -122,12 +125,12 @@ module Hedgehog.Internal.Property (
   ) where
 
 import           Control.Applicative (Alternative(..))
+import           Control.DeepSeq (NFData, rnf)
 import           Control.Monad (MonadPlus(..))
 import           Control.Monad.Base (MonadBase(..))
 import           Control.Monad.Catch (MonadThrow(..), MonadCatch(..))
 import           Control.Monad.Catch (SomeException(..), displayException)
 import           Control.Monad.Error.Class (MonadError(..))
-import           Control.Monad.Fail (MonadFail (..))
 import qualified Control.Monad.Fail as Fail
 import           Control.Monad.IO.Class (MonadIO(..))
 import           Control.Monad.Morph (MFunctor(..))
@@ -152,13 +155,13 @@ import qualified Control.Monad.Trans.Writer.Lazy as Lazy
 import qualified Control.Monad.Trans.Writer.Strict as Strict
 
 import qualified Data.Char as Char
+import           Data.Functor (($>))
 import           Data.Functor.Identity (Identity(..))
 import           Data.Int (Int64)
 import           Data.Map (Map)
 import qualified Data.Map.Strict as Map
 import           Data.Number.Erf (invnormcdf)
 import qualified Data.List as List
-import           Data.Semigroup (Semigroup(..))
 import           Data.String (IsString)
 import           Data.Ratio ((%))
 import           Data.Typeable (typeOf)
@@ -167,11 +170,13 @@ import           Hedgehog.Internal.Distributive
 import           Hedgehog.Internal.Exception
 import           Hedgehog.Internal.Gen (Gen, GenT)
 import qualified Hedgehog.Internal.Gen as Gen
+import           Hedgehog.Internal.Prelude
 import           Hedgehog.Internal.Show
 import           Hedgehog.Internal.Source
 
 import           Language.Haskell.TH.Syntax (Lift)
 
+
 ------------------------------------------------------------------------
 
 -- | A property test, along with some configurable limits like how many times
@@ -700,9 +705,17 @@ failDiff x y =
 --   message.
 --
 failException :: (MonadTest m, HasCallStack) => SomeException -> m a
-failException (SomeException x) =
+failException x =
   withFrozenCallStack $
-    failWith Nothing $ unlines [
+    failExceptionWith [] x
+
+-- | Fails with an error which renders the given messages, the type of an exception,
+--   and its error message.
+--
+failExceptionWith :: (MonadTest m, HasCallStack) => [String] -> SomeException -> m a
+failExceptionWith messages (SomeException x) =
+  withFrozenCallStack
+    failWith Nothing $ unlines $ messages <> [
         "━━━ Exception (" ++ show (typeOf x) ++ ") ━━━"
       , List.dropWhileEnd Char.isSpace (displayException x)
       ]
@@ -779,6 +792,17 @@ eval :: (MonadTest m, HasCallStack) => a -> m a
 eval x =
   either (withFrozenCallStack failException) pure (tryEvaluate x)
 
+-- | Fails the test if the value throws an exception when evaluated to
+--   normal form (NF).
+--
+evalNF :: (MonadTest m, NFData a, HasCallStack) => a -> m a
+evalNF x =
+  let
+    messages =
+      ["━━━ Value could not be evaluated to normal form ━━━"]
+  in
+    either (withFrozenCallStack (failExceptionWith messages)) pure (tryEvaluate (rnf x)) $> x
+
 -- | Fails the test if the action throws an exception.
 --
 --   /The benefit of using this over simply letting the exception bubble up is/
@@ -1107,7 +1131,7 @@ coverageSuccess tests =
 
 coverageFailures :: TestCount -> Coverage CoverCount -> [Label CoverCount]
 coverageFailures tests (Coverage kvs) =
-  filter (not . labelCovered tests) (Map.elems kvs)
+  List.filter (not . labelCovered tests) (Map.elems kvs)
 
 -- | Is true when the test coverage satisfies the specified 'Confidence'
 --   contstraint for all 'Coverage CoverCount's
