@@ -25,6 +25,9 @@
 -- 2. Nikos Baxevanis
 --    https://github.com/moodmosaic/SplitMix/blob/master/SplitMix.hs
 --
+
+#include "MachDeps.h"
+
 module Hedgehog.Internal.Seed (
     Seed(..)
   , random
@@ -48,7 +51,11 @@ import           Control.Monad.IO.Class (MonadIO(..))
 
 import           Data.Bifunctor (first)
 import           Data.Bits ((.|.), xor, shiftR, popCount)
-import           Data.Int (Int32, Int64)
+#if (SIZEOF_HSINT == 8)
+import           Data.Int (Int64)
+#else
+import           Data.Int (Int32)
+#endif
 import           Data.Time.Clock.POSIX (getPOSIXTime)
 import           Data.IORef (IORef)
 import qualified Data.IORef as IORef
@@ -100,7 +107,7 @@ random =
 --
 from :: Word64 -> Seed
 from x =
-  Seed x goldenGamma
+  Seed (mix64 x) (mixGamma (x + goldenGamma))
 
 -- | A predefined gamma value's needed for initializing the "root" instances of
 --   'Seed'. That is, instances not produced by splitting an already existing
@@ -192,7 +199,7 @@ mixGamma x =
     y = mix64variant13 x .|. 1
     n = popCount $ y `xor` (y `shiftR` 1)
   in
-    if n >= 24 then
+    if n < 24 then
       y `xor` 0xaaaaaaaaaaaaaaaa
     else
       y
@@ -200,8 +207,6 @@ mixGamma x =
 ------------------------------------------------------------------------
 -- RandomGen instances
 
-#include "MachDeps.h"
-
 #if (SIZEOF_HSINT == 8)
 instance RandomGen Seed where
   next =
