@@ -11,6 +11,9 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE UndecidableInstances #-} -- MonadBase
+#if __GLASGOW_HASKELL__ < 802
+{-# OPTIONS_GHC -fno-warn-incomplete-patterns #-}
+#endif
 module Hedgehog.Internal.Tree (
     Tree
   , pattern Tree
@@ -86,7 +89,9 @@ type Tree =
 pattern Tree :: NodeT Identity a -> Tree a
 pattern Tree node =
   TreeT (Identity node)
+#if __GLASGOW_HASKELL__ >= 802
 {-# COMPLETE Tree #-}
+#endif
 
 -- | An effectful tree, each node in the tree can have an effect before it is
 --   produced.
@@ -105,7 +110,9 @@ instance MonadBaseControl b m => MonadBaseControl b (TreeT m) where
 --
 type Node =
   NodeT Identity
+#if __GLASGOW_HASKELL__ >= 802
 {-# COMPLETE Node #-}
+#endif
 
 -- | Pattern to ease construction / deconstruction of pure nodes.
 --
@@ -330,11 +337,30 @@ splits xs0 =
   in
     go (List.inits xs0) xs0
 
-dropOne :: Monad m => [NodeT m a] -> [TreeT m [a]]
-dropOne ts = do
-  (xs, _y, zs) <- splits ts
-  pure . TreeT . pure $
-    interleave (xs ++ zs)
+-- | @removes n@ computes all ways we can remove chunks of size @n@ from a list
+--
+-- Examples
+--
+-- > removes 1 [1..3] == [[2,3],[1,3],[1,2]]
+-- > removes 2 [1..4] == [[3,4],[1,2]]
+-- > removes 2 [1..5] == [[3,4,5],[1,2,5],[1,2,3,4]]
+-- > removes 3 [1..5] == [[4,5],[1,2,3]]
+--
+-- Note that the last chunk we delete might have fewer elements than @n@.
+removes :: forall a. Int -> [a] -> [[a]]
+removes k = \xs -> go xs
+  where
+    go :: [a] -> [[a]]
+    go [] = []
+    go xs = xs2 : map (xs1 ++) (go xs2)
+      where
+        (xs1, xs2) = splitAt k xs
+
+dropSome :: Monad m => [NodeT m a] -> [TreeT m [a]]
+dropSome ts = do
+  n   <- takeWhile (> 0) $ iterate (`div` 2) (length ts)
+  ts' <- removes n ts
+  pure . TreeT . pure $ interleave ts'
 
 shrinkOne :: Monad m => [NodeT m a] -> [TreeT m [a]]
 shrinkOne ts = do
@@ -349,7 +375,7 @@ interleave :: forall m a. Monad m => [NodeT m a] -> NodeT m [a]
 interleave ts =
   NodeT (fmap nodeValue ts) $
     concat [
-        dropOne ts
+        dropSome ts
       , shrinkOne ts
       ]
 
