@@ -3,13 +3,24 @@ module Hedgehog.Internal.Tripping (
     tripping
   ) where
 
-import           Hedgehog.Internal.Property
-import           Hedgehog.Internal.Show
-import           Hedgehog.Internal.Source
+import           Hedgehog.Internal.Property (MonadTest, Diff(..), success, failWith)
+import           Hedgehog.Internal.Show (valueDiff, mkValue, showPretty)
+import           Hedgehog.Internal.Source (HasCallStack, withFrozenCallStack)
 
 
 -- | Test that a pair of encode / decode functions are compatible.
 --
+-- Given a printer from some type 'a -> b', and a parser with a
+-- potential failure case 'b -> f a'. Ensure that a valid 'a' round
+-- trips through the "print" and "parse" to yield the same 'a'.
+--
+-- For example, types __should__ have tripping 'Read' and 'Show'
+-- instances.
+--
+-- @
+-- trippingShowRead :: (Show a, Read a, Eq a, MonadTest m) => a -> m ()
+-- trippingShowRead a = tripping a show readEither
+-- @
 tripping ::
      (MonadTest m, Applicative f, Show b, Show (f a), Eq (f a), HasCallStack)
   => a
@@ -45,7 +56,8 @@ tripping x encode decode =
         Just diff ->
           withFrozenCallStack $
             failWith
-              (Just $ Diff "━━━ " "- Original" "/" "+ Roundtrip" " ━━━" diff) $
+              (Just $
+                Diff "━━━ " "- Original" ") (" "+ Roundtrip" " ━━━" diff) $
               unlines [
                   "━━━ Intermediate ━━━"
                 , showPretty i
